module FreqCounter(
        input  clk_12MHz,
        output uart_tx,
        input  clk_in,
        output clk_12MHz_tp,
        output uart_tx_tp,
    );

    // Test points
    assign clk_12MHz_tp = clk_12MHz;
    assign uart_tx_tp   = uart_tx;

    // Counter
    reg         active;
    wire        clk_gated = clk_in & active;
    reg  [31:0] counter;
    always @(posedge clk_gated)
        counter <= counter + 1;

    // Readout timer
    reg  [31:0] timeout;
    wire        timeout_strobe = (timeout == 0);
    always @(posedge clk_12MHz)
        if(timeout == 0)
            timeout <= 12_000_001;
        else
            timeout <= timeout - 1;

    // UART and readout FSM
    reg   [7:0] tx_data;
    reg         tx_ready;
    wire        tx_ack;
    UART #(
        .FREQ(12_000_000),
        .BAUD(115_200)
    ) uart (
        .reset(1'b1),
        .clk(clk_12MHz),
        .tx_o(uart_tx),
        .tx_data_i(tx_data),
        .tx_ready_i(tx_ready),
        .tx_ack_o(tx_ack)
    );

    localparam S_IDLE  = 0;
    localparam S_SYNC  = 1;
    localparam S_SEND  = 2;
    localparam S_WAIT  = 3;

    reg   [3:0] state  = S_IDLE;
    reg   [1:0] byteno = 2'd0;
    reg  [31:0] buffer;
    always @(posedge clk_12MHz)
        case(state)
            S_IDLE: begin
                if(timeout_strobe) begin
                    active <= 0;
                    state <= S_SYNC;
                end
            end
            S_SYNC: begin
                buffer <= counter;
                active <= 1;
                state <= S_SEND;
            end
            S_SEND: begin
                tx_data <= buffer >> (byteno * 8);
                tx_ready <= 1;
                if(!tx_ack)
                    state <= S_WAIT;
            end
            S_WAIT: begin
                tx_ready <= 0;
                if(tx_ack) begin
                    byteno <= byteno + 1;
                    if(byteno == 3)
                        state <= S_IDLE;
                    else
                        state <= S_SEND;
                end
            end
        endcase

endmodule
